/* This file is part of GNU epsilon, a functional language implementation

Copyright (C) 2002, 2003 Luca Saiu

GNU epsilon is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2, or (at your
option) any later version.

GNU epsilon is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with epsilon; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA. */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#include "eam_types.h"
#include "runtime.h"
#include "eam.h"


void
fatal (char *message)
{
  fprintf (stderr, "FATAL: %s\n", message);
  exit (-1);
}

void *
fast_lookup_array (void *a, void *i)
{
  return ((void **) a)[(int) i];
}

void
check_not_null (void *x)
{
  if (x == NULL)
    fatal ("check_not_null(): parameter is null");
}

void
check_array_access_left (int i)
{
  if (i < 0)
    fatal ("check_array_access_left(): out of bounds");
}

void
check_array_access_right (int *a, int i)
{
  if (((int *) a)[0] >= (int) i)
    fatal ("check_array_access_right(): out of bounds");
}

void *
left_safe_lookup_array (void *a, void *i)
{
  check_array_access_left ((int) i);

  return fast_lookup_array (a, i);
}

void *
right_safe_lookup_array (void *a, void *i)
{
  check_array_access_right ((int *) a, (int) i);

  return fast_lookup_array (a, i);
}

void *
safe_lookup_array (void *a, void *i)
{
  check_array_access_left ((int) i);
  check_array_access_right ((int *) a, (int) i);

  return fast_lookup_array (a, i);
}

void *
fast_car (void *x)
{
  return ((void **) x)[0];
}

void *
fast_cdr (void *x)
{
  return ((void **) x)[1];
}

void *
safe_car (void *x)
{
  check_not_null (x);

  return fast_car (x);
}

void *
safe_cdr (void *x)
{
  check_not_null (x);

  return fast_cdr (x);
}

/*
void *a, *b;

void
test ()
{
  void **x;
  void *y = a, *z = b;
  void **stack;
  void *top;
  int undertop_stack_pointer;

  //asm ("# ---");
  ASSIGN_CONS (x, y, z);
  asm ("# ---");
  POP_MULTIPLE (50);
  asm ("# ---");
  printf ("%p %i %p\n", stack, undertop_stack_pointer, top);
}
*/
